# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.__internal__.feature_column namespace.
"""

import sys as _sys

from tensorflow.python.feature_column.feature_column_v2 import DenseColumn
from tensorflow.python.feature_column.feature_column_v2 import FeatureColumn
from tensorflow.python.feature_column.feature_column_v2 import FeatureTransformationCache
from tensorflow.python.feature_column.feature_column_v2 import SequenceDenseColumn
from tensorflow.python.feature_column.feature_column_v2 import _StateManagerImpl as StateManager
from tensorflow.python.feature_column.serialization import deserialize_feature_column
from tensorflow.python.feature_column.serialization import serialize_feature_column